@@ -22,16 +22,24 @@
  */
 
 #include <cairo.h>
+
+#if PY_VERSION_HEX < 0x03000000
 #include <pycairo.h>
+#else
+#include <pycairo/py3cairo.h>
+#endif
+
 Pycairo_CAPI_t *Pycairo_CAPI;
 
 #include "pygi-foreign.h"
 
+#include <pyglib-python-compat.h>
+
 PyObject *
 cairo_context_to_arg (PyObject       *value,
                       GITypeInfo     *type_info,
                       GITransfer      transfer,
-                      GArgument      *arg)
+                      GIArgument      *arg)
 {
     cairo_t *cr;
 
@@ -47,7 +55,7 @@ cairo_context_to_arg (PyObject       *value,
 }
 
 PyObject *
-cairo_context_from_arg (GITypeInfo *type_info, GArgument  *arg)
+cairo_context_from_arg (GITypeInfo *type_info, GIArgument  *arg)
 {
     cairo_t *context = (cairo_t*) arg;
 
@@ -57,10 +65,10 @@ cairo_context_from_arg (GITypeInfo *type_info, GArgument  *arg)
 }
 
 PyObject *
-cairo_context_release_arg (GITransfer  transfer, GITypeInfo *type_info,
-                           GArgument  *arg)
+cairo_context_release (GIBaseInfo *base_info,
+                       gpointer    struct_)
 {
-    cairo_destroy ( (cairo_t*) arg->v_pointer);
+    cairo_destroy ( (cairo_t*) struct_);
     Py_RETURN_NONE;
 }
 
@@ -69,7 +77,7 @@ PyObject *
 cairo_surface_to_arg (PyObject       *value,
                       GITypeInfo     *type_info,
                       GITransfer      transfer,
-                      GArgument      *arg)
+                      GIArgument      *arg)
 {
     cairo_surface_t *surface;
 
@@ -86,7 +94,7 @@ cairo_surface_to_arg (PyObject       *value,
 }
 
 PyObject *
-cairo_surface_from_arg (GITypeInfo *type_info, GArgument  *arg)
+cairo_surface_from_arg (GITypeInfo *type_info, GIArgument  *arg)
 {
     cairo_surface_t *surface = (cairo_surface_t*) arg;
 
@@ -96,36 +104,31 @@ cairo_surface_from_arg (GITypeInfo *type_info, GArgument  *arg)
 }
 
 PyObject *
-cairo_surface_release_arg (GITransfer  transfer, GITypeInfo *type_info,
-                           GArgument  *arg)
+cairo_surface_release (GIBaseInfo *base_info,
+                       gpointer    struct_)
 {
-    cairo_surface_destroy ( (cairo_surface_t*) arg->v_pointer);
+    cairo_surface_destroy ( (cairo_surface_t*) struct_);
     Py_RETURN_NONE;
 }
 
-PyMODINIT_FUNC
-init_gi_cairo (void)
+
+static PyMethodDef _gi_cairo_functions[] = {};
+PYGLIB_MODULE_START(_gi_cairo, "_gi_cairo")
 {
-    PyObject *m;
-
-    m = Py_InitModule ("_gi_cairo", NULL);
-    if (m == NULL) {
-        return;
-    }
-
     Pycairo_IMPORT;
     if (Pycairo_CAPI == NULL)
-        return;
+        return 0;
 
     pygi_register_foreign_struct ("cairo",
                                   "Context",
                                   cairo_context_to_arg,
                                   cairo_context_from_arg,
-                                  cairo_context_release_arg);
+                                  cairo_context_release);
 
     pygi_register_foreign_struct ("cairo",
                                   "Surface",
                                   cairo_surface_to_arg,
                                   cairo_surface_from_arg,
-                                  cairo_surface_release_arg);
+                                  cairo_surface_release);
 }
+PYGLIB_MODULE_END;
