@@ -50,7 +50,7 @@ def Constructor(info):
     def constructor(cls, *args):
         cls_name = info.get_container().get_name()
         if cls.__name__ != cls_name:
-            raise TypeError, '%s constructor cannot be used to create instances of a subclass' % cls_name
+            raise TypeError('%s constructor cannot be used to create instances of a subclass' % cls_name)
         return info.invoke(cls, *args)
 
     constructor.__info__ = info
@@ -105,10 +105,14 @@ class MetaClassHelper(object):
                             'the method do_%s()' % (base.__info__.get_namespace(),
                                                     base.__info__.get_name(),
                                                     vfunc_info.get_name()))
-                elif vfunc is not None and not \
-                        is_function_in_classes(vfunc.im_func, cls.__bases__):
-                    hook_up_vfunc_implementation(vfunc_info, cls.__gtype__,
-                                                 vfunc)
+                elif vfunc is not None:
+                    function = vfunc
+                    if sys.version_info < (3, 0):
+                        function = vfunc.im_func
+ 
+                    if not is_function_in_classes(function, cls.__bases__):
+                        hook_up_vfunc_implementation(vfunc_info, cls.__gtype__,
+                                                     vfunc)
 
 def is_function_in_classes(function, classes):
     for klass in classes:
@@ -161,6 +165,7 @@ class StructMeta(type, MetaClassHelper):
 
 def override(type_):
     g_type = type_.__info__.get_g_type()
+    assert g_type != gobject.TYPE_NONE
     if g_type != gobject.TYPE_INVALID:
         g_type.pytype = type_
     return type_
