@@ -317,9 +317,32 @@ pygobject_init(int req_major, int req_minor, int req_micro)
             Py_XDECREF(type);
             Py_XDECREF(value);
             Py_XDECREF(traceback);
+
+
+#if PY_VERSION_HEX < 0x03000000
             PyErr_Format(PyExc_ImportError,
                          "could not import gobject (error was: %s)",
                          PyString_AsString(py_orig_exc));
+#else
+            {
+                /* Can not use PyErr_Format because it doesn't have
+                 * a format string for dealing with PyUnicode objects
+                 * like PyUnicode_FromFormat has
+                 */
+                PyObject *errmsg = PyUnicode_FromFormat("could not import gobject (error was: %U)",
+                                                        py_orig_exc);
+
+                if (errmsg) {
+                   PyErr_SetObject(PyExc_ImportError,
+                                   errmsg);
+                   Py_DECREF(errmsg);
+                }
+                /* if errmsg is NULL then we might have OOM
+                 * PyErr should already be set and trying to
+                 * return our own error would be futile
+                 */
+            }
+#endif
             Py_DECREF(py_orig_exc);
         } else {
             PyErr_SetString(PyExc_ImportError,
