@@ -1,22 +1,22 @@
 # -*- Mode: Python; py-indent-offset: 4 -*-
+# coding=utf-8
 # vim: tabstop=4 shiftwidth=4 expandtab
 
+import sys
 import pygtk
 pygtk.require("2.0")
 
 import unittest
 from gi.repository import GObject
 
-from datetime import datetime
-
-import sys
-sys.path.insert(0, "../")
-
 import gobject
 from gi.repository import GIMarshallingTests
 
+if sys.version_info < (3, 0):
+    CONSTANT_UTF8 = "const \xe2\x99\xa5 utf8"
+else:
+    CONSTANT_UTF8 = "const ♥ utf8"
 
-CONSTANT_UTF8 = "const \xe2\x99\xa5 utf8"
 CONSTANT_NUMBER = 42
 
 
@@ -588,25 +588,6 @@ class TestDouble(unittest.TestCase):
         self.assertAlmostEquals(self.MIN, GIMarshallingTests.double_inout(Number(self.MAX)))
 
 
-class TestTimeT(unittest.TestCase):
-
-    DATETIME = datetime.fromtimestamp(1234567890)
-
-    def test_time_t_return(self):
-        self.assertEquals(self.DATETIME, GIMarshallingTests.time_t_return())
-
-    def test_time_t_in(self):
-        GIMarshallingTests.time_t_in(self.DATETIME)
-
-        self.assertRaises(TypeError, GIMarshallingTests.time_t_in, "self.DATETIME")
-
-    def test_time_t_out(self):
-        self.assertEquals(self.DATETIME, GIMarshallingTests.time_t_out())
-
-    def test_time_t_inout(self):
-        self.assertEquals(datetime.fromtimestamp(0), GIMarshallingTests.time_t_inout(self.DATETIME))
-
-
 class TestGType(unittest.TestCase):
 
     def test_gtype_return(self):
@@ -638,9 +619,6 @@ class TestUtf8(unittest.TestCase):
         self.assertRaises(TypeError, GIMarshallingTests.utf8_none_in, CONSTANT_NUMBER)
         self.assertRaises(TypeError, GIMarshallingTests.utf8_none_in, None)
 
-    def test_utf8_full_in(self):
-        GIMarshallingTests.utf8_full_in(CONSTANT_UTF8)
-
     def test_utf8_none_out(self):
         self.assertEquals(CONSTANT_UTF8, GIMarshallingTests.utf8_none_out())
 
@@ -775,12 +753,6 @@ class TestGArray(unittest.TestCase):
     def test_garray_utf8_none_in(self):
         GIMarshallingTests.garray_utf8_none_in(Sequence(['0', '1', '2']))
 
-    def test_garray_utf8_container_in(self):
-        GIMarshallingTests.garray_utf8_container_in(Sequence(['0', '1', '2']))
-
-    def test_garray_utf8_full_in(self):
-        GIMarshallingTests.garray_utf8_full_in(Sequence(['0', '1', '2']))
-
     def test_garray_utf8_none_out(self):
         self.assertEquals(['0', '1', '2'], GIMarshallingTests.garray_utf8_none_out())
 
@@ -825,12 +797,6 @@ class TestGList(unittest.TestCase):
     def test_glist_utf8_none_in(self):
         GIMarshallingTests.glist_utf8_none_in(Sequence(('0', '1', '2')))
 
-    def test_glist_utf8_container_in(self):
-        GIMarshallingTests.glist_utf8_container_in(Sequence(('0', '1', '2')))
-
-    def test_glist_utf8_full_in(self):
-        GIMarshallingTests.glist_utf8_full_in(Sequence(('0', '1', '2')))
-
     def test_glist_utf8_none_out(self):
         self.assertEquals(['0', '1', '2'], GIMarshallingTests.glist_utf8_none_out())
 
@@ -875,12 +841,6 @@ class TestGSList(unittest.TestCase):
     def test_gslist_utf8_none_in(self):
         GIMarshallingTests.gslist_utf8_none_in(Sequence(('0', '1', '2')))
 
-    def test_gslist_utf8_container_in(self):
-        GIMarshallingTests.gslist_utf8_container_in(Sequence(('0', '1', '2')))
-
-    def test_gslist_utf8_full_in(self):
-        GIMarshallingTests.gslist_utf8_full_in(Sequence(('0', '1', '2')))
-
     def test_gslist_utf8_none_out(self):
         self.assertEquals(['0', '1', '2'], GIMarshallingTests.gslist_utf8_none_out())
 
@@ -926,12 +886,6 @@ class TestGHashTable(unittest.TestCase):
     def test_ghashtable_utf8_none_in(self):
         GIMarshallingTests.ghashtable_utf8_none_in({'-1': '1', '0': '0', '1': '-1', '2': '-2'})
 
-    def test_ghashtable_utf8_container_in(self):
-        GIMarshallingTests.ghashtable_utf8_container_in({'-1': '1', '0': '0', '1': '-1', '2': '-2'})
-
-    def test_ghashtable_utf8_full_in(self):
-        GIMarshallingTests.ghashtable_utf8_full_in({'-1': '1', '0': '0', '1': '-1', '2': '-2'})
-
     def test_ghashtable_utf8_none_out(self):
         self.assertEquals({'-1': '1', '0': '0', '1': '-1', '2': '-2'}, GIMarshallingTests.ghashtable_utf8_none_out())
 
@@ -995,8 +949,9 @@ class TestEnum(unittest.TestCase):
 
     def test_enum_in(self):
         GIMarshallingTests.enum_in(GIMarshallingTests.Enum.VALUE3)
+        GIMarshallingTests.enum_in(42)
 
-        self.assertRaises(TypeError, GIMarshallingTests.enum_in, 42)
+        self.assertRaises(TypeError, GIMarshallingTests.enum_in, 43)
         self.assertRaises(TypeError, GIMarshallingTests.enum_in, 'GIMarshallingTests.Enum.VALUE3')
 
     def test_enum_out(self):
@@ -1021,8 +976,9 @@ class TestGEnum(unittest.TestCase):
 
     def test_genum_in(self):
         GIMarshallingTests.genum_in(GIMarshallingTests.GEnum.VALUE3)
+        GIMarshallingTests.genum_in(42)
 
-        self.assertRaises(TypeError, GIMarshallingTests.genum_in, 42)
+        self.assertRaises(TypeError, GIMarshallingTests.genum_in, 43)
         self.assertRaises(TypeError, GIMarshallingTests.genum_in, 'GIMarshallingTests.GEnum.VALUE3')
 
     def test_genum_out(self):
@@ -1097,7 +1053,7 @@ class TestStructure(unittest.TestCase):
         self.assertEquals(None, struct.pointer)
 
     def test_simple_struct_return(self):
-        struct = GIMarshallingTests.simple_struct_return()
+        struct = GIMarshallingTests.simple_struct_returnv()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.SimpleStruct))
         self.assertEquals(6, struct.long_)
@@ -1110,40 +1066,17 @@ class TestStructure(unittest.TestCase):
         struct.long_ = 6
         struct.int8 = 7
 
-        GIMarshallingTests.simple_struct_in(struct)
+        GIMarshallingTests.SimpleStruct.inv(struct)
 
         del struct
 
         struct = GIMarshallingTests.NestedStruct()
 
-        self.assertRaises(TypeError, GIMarshallingTests.simple_struct_in, struct)
+        self.assertRaises(TypeError, GIMarshallingTests.SimpleStruct.inv, struct)
 
         del struct
 
-        self.assertRaises(TypeError, GIMarshallingTests.simple_struct_in, None)
-
-    def test_simple_struct_out(self):
-        struct = GIMarshallingTests.simple_struct_out()
-
-        self.assertTrue(isinstance(struct, GIMarshallingTests.SimpleStruct))
-        self.assertEquals(6, struct.long_)
-        self.assertEquals(7, struct.int8)
-
-        del struct
-
-    def test_simple_struct_inout(self):
-        in_struct = GIMarshallingTests.SimpleStruct()
-        in_struct.long_ = 6
-        in_struct.int8 = 7
-
-        out_struct = GIMarshallingTests.simple_struct_inout(in_struct)
-
-        self.assertTrue(isinstance(out_struct, GIMarshallingTests.SimpleStruct))
-        self.assertEquals(7, out_struct.long_)
-        self.assertEquals(6, out_struct.int8)
-
-        del in_struct
-        del out_struct
+        self.assertRaises(TypeError, GIMarshallingTests.SimpleStruct.inv, None)
 
     def test_simple_struct_method(self):
         struct = GIMarshallingTests.SimpleStruct()
@@ -1166,7 +1099,7 @@ class TestStructure(unittest.TestCase):
         del struct
 
     def test_pointer_struct_return(self):
-        struct = GIMarshallingTests.pointer_struct_return()
+        struct = GIMarshallingTests.pointer_struct_returnv()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.PointerStruct))
         self.assertEquals(42, struct.long_)
@@ -1177,30 +1110,10 @@ class TestStructure(unittest.TestCase):
         struct = GIMarshallingTests.PointerStruct()
         struct.long_ = 42
 
-        GIMarshallingTests.pointer_struct_in(struct)
+        struct.inv()
 
         del struct
 
-    def test_pointer_struct_out(self):
-        struct = GIMarshallingTests.pointer_struct_out()
-
-        self.assertTrue(isinstance(struct, GIMarshallingTests.PointerStruct))
-        self.assertEquals(42, struct.long_)
-
-        del struct
-
-    def test_pointer_struct_inout(self):
-        in_struct = GIMarshallingTests.PointerStruct()
-        in_struct.long_ = 42
-
-        out_struct = GIMarshallingTests.pointer_struct_inout(in_struct)
-
-        self.assertTrue(isinstance(out_struct, GIMarshallingTests.PointerStruct))
-        self.assertEquals(0, out_struct.long_)
-
-        del in_struct
-        del out_struct
-
     def test_boxed_struct(self):
         self.assertTrue(issubclass(GIMarshallingTests.BoxedStruct, GObject.GBoxed))
 
@@ -1228,7 +1141,7 @@ class TestStructure(unittest.TestCase):
         del struct
 
     def test_boxed_struct_return(self):
-        struct = GIMarshallingTests.boxed_struct_return()
+        struct = GIMarshallingTests.boxed_struct_returnv()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.BoxedStruct))
         self.assertEquals(42, struct.long_)
@@ -1240,7 +1153,7 @@ class TestStructure(unittest.TestCase):
         struct = GIMarshallingTests.BoxedStruct()
         struct.long_ = 42
 
-        GIMarshallingTests.boxed_struct_in(struct)
+        struct.inv()
 
         del struct
 
@@ -1276,7 +1189,7 @@ class TestStructure(unittest.TestCase):
         del new_union
 
     def test_union_return(self):
-        union = GIMarshallingTests.union_return()
+        union = GIMarshallingTests.union_returnv()
 
         self.assertTrue(isinstance(union, GIMarshallingTests.Union))
         self.assertEquals(42, union.long_)
@@ -1287,30 +1200,10 @@ class TestStructure(unittest.TestCase):
         union = GIMarshallingTests.Union()
         union.long_ = 42
 
-        GIMarshallingTests.union_in(union)
+        union.inv()
 
         del union
 
-    def test_union_out(self):
-        union = GIMarshallingTests.union_out()
-
-        self.assertTrue(isinstance(union, GIMarshallingTests.Union))
-        self.assertEquals(42, union.long_)
-
-        del union
-
-    def test_union_inout(self):
-        in_union = GIMarshallingTests.Union()
-        in_union.long_ = 42
-
-        out_union = GIMarshallingTests.union_inout(in_union)
-
-        self.assertTrue(isinstance(out_union, GIMarshallingTests.Union))
-        self.assertEquals(0, out_union.long_)
-
-        del in_union
-        del out_union
-
     def test_union_method(self):
         union = GIMarshallingTests.Union()
         union.long_ = 42
@@ -1386,49 +1279,44 @@ class TestGObject(unittest.TestCase):
 #        self.assertEquals(object_.int_, 42)
 
     def test_object_none_return(self):
-        object_ = GIMarshallingTests.object_none_return()
+        object_ = GIMarshallingTests.Object.none_return()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 2)
 
     def test_object_full_return(self):
-        object_ = GIMarshallingTests.object_full_return()
+        object_ = GIMarshallingTests.Object.full_return()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 1)
 
     def test_object_none_in(self):
         object_ = GIMarshallingTests.Object(int = 42)
-        GIMarshallingTests.object_none_in(object_)
+        GIMarshallingTests.Object.none_in(object_)
         self.assertEquals(object_.__grefcount__, 1)
 
         object_ = GIMarshallingTests.SubObject(int = 42)
-        GIMarshallingTests.object_none_in(object_)
+        GIMarshallingTests.Object.none_in(object_)
 
         object_ = GObject.GObject()
-        self.assertRaises(TypeError, GIMarshallingTests.object_none_in, object_)
+        self.assertRaises(TypeError, GIMarshallingTests.Object.none_in, object_)
 
-        self.assertRaises(TypeError, GIMarshallingTests.object_none_in, None)
-
-    def test_object_full_in(self):
-        object_ = GIMarshallingTests.Object(int = 42)
-        GIMarshallingTests.object_full_in(object_)
-        self.assertEquals(object_.__grefcount__, 1)
+        self.assertRaises(TypeError, GIMarshallingTests.Object.none_in, None)
 
     def test_object_none_out(self):
-        object_ = GIMarshallingTests.object_none_out()
+        object_ = GIMarshallingTests.Object.none_out()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 2)
 
-        new_object = GIMarshallingTests.object_none_out()
+        new_object = GIMarshallingTests.Object.none_out()
         self.assertTrue(new_object is object_)
 
     def test_object_full_out(self):
-        object_ = GIMarshallingTests.object_full_out()
+        object_ = GIMarshallingTests.Object.full_out()
         self.assertTrue(isinstance(object_, GIMarshallingTests.Object))
         self.assertEquals(object_.__grefcount__, 1)
 
     def test_object_none_inout(self):
         object_ = GIMarshallingTests.Object(int = 42)
-        new_object = GIMarshallingTests.object_none_inout(object_)
+        new_object = GIMarshallingTests.Object.none_inout(object_)
 
         self.assertTrue(isinstance(new_object, GIMarshallingTests.Object))
 
@@ -1437,20 +1325,20 @@ class TestGObject(unittest.TestCase):
         self.assertEquals(object_.__grefcount__, 1)
         self.assertEquals(new_object.__grefcount__, 2)
 
-        new_new_object = GIMarshallingTests.object_none_inout(object_)
+        new_new_object = GIMarshallingTests.Object.none_inout(object_)
         self.assertTrue(new_new_object is new_object)
 
-        GIMarshallingTests.object_none_inout(GIMarshallingTests.SubObject(int = 42))
+        GIMarshallingTests.Object.none_inout(GIMarshallingTests.SubObject(int = 42))
 
     def test_object_full_inout(self):
         object_ = GIMarshallingTests.Object(int = 42)
-        new_object = GIMarshallingTests.object_full_inout(object_)
+        new_object = GIMarshallingTests.Object.full_inout(object_)
 
         self.assertTrue(isinstance(new_object, GIMarshallingTests.Object))
 
         self.assertFalse(object_ is new_object)
 
-        self.assertEquals(object_.__grefcount__, 1)
+        self.assertEquals(object_.__grefcount__, 2)
         self.assertEquals(new_object.__grefcount__, 1)
 
 # FIXME: Doesn't actually return the same object.
@@ -1594,7 +1482,7 @@ class TestOverrides(unittest.TestCase):
         del struct
 
         # Test that the overrides wrapper has been registered.
-        struct = GIMarshallingTests.overrides_struct_return()
+        struct = GIMarshallingTests.overrides_struct_returnv()
 
         self.assertTrue(isinstance(struct, GIMarshallingTests.OverridesStruct))
 
@@ -1615,7 +1503,7 @@ class TestOverrides(unittest.TestCase):
         self.assertEquals(6, object_.method())
 
         # Test that the overrides wrapper has been registered.
-        object_ = GIMarshallingTests.overrides_object_return()
+        object_ = GIMarshallingTests.OverridesObject.returnv()
 
         self.assertTrue(isinstance(object_, GIMarshallingTests.OverridesObject))
 
